#!./sdbsiod -f

; This can be any valid sdb url
(define url "sqlite3:db=test")
;(define url "mysql:db=test:uid=root")
;(define url "oracle:uid=system/manager@test")
;(define url "postgres:host=localhost:db=test")

; Uncomment to speed up
; Without this, the database will be opened and closed with each query
(define url (sdb_open url))

(puts (string-append "url = " url "\n"))

; Safely convert anything to a number
(define (any->number p)
 (cond ((number? p) p)
       ((pair? p) (any->number (car p)))
       ((string? p) (string->number p))
       (t 0)))

; Safely convert anything to a string
(define (any->string p)
 (cond ((string? p) p)
       ((pair? p) (any->string (car p)))
       ((number? p) (number->string p))
       ((null? p) "nil")
       (t "unknown")))


; Display the concatentation of the arguments regardless of type
(define (echo . str)
 (puts (apply string-append (mapcar any->string str))))


; This little function makes the interface more convenient
(define (sdb url . query)
 (sdb_query url (apply string-append query) my_cb nil))

(define (my_cb n p closure)
 (if (null? p)
     (puts "\n")
     (begin (puts (car p))
            (puts " ")
            (my_cb (- n 1) (cdr p) closure)))
 0)


; And if you only want to get a single row, use sdb_get
(define (sdb_get url . query)
 (let ((result (cons nil nil)))
  (sdb_query url (apply string-append (mapcar any->string query)) sdb_get_cb result)
  (car result)))

(define (sdb_get_cb n p closure)
 (set-car! closure p)
 0)


; Create a table and place some data into it
(sdb url "drop table b")
(sdb url "create table b (a integer, b integer)")
(sdb url "insert into b values (1, 2)")
(sdb url "select * from b")
(sdb url "insert into b select * from b")
(sdb url "insert into b select * from b")
(sdb url "insert into b select * from b")
(sdb url "insert into b select * from b")
(sdb url "select count(*) from b")


; Another table
(sdb url "drop table mail")
(sdb url "create table mail (first varchar(255),
                             last varchar(255),
                             mail varchar(255))")
(sdb url "insert into mail values (\"Ulric\", \"Eriksson\", \"ulric@siag.nu\")")
(sdb url "insert into mail values (\"Ulva\", \"Eriksson\", \"ulva@siag.nu\")")
(sdb url "insert into mail values (\"Majken\", \"Eriksson\", \"majken@siag.nu\")")
(sdb url "select mail from mail where first like \"Ul%\"")


; A function which displays the name associated with a mail address
(define (mailuser address)
 (let ((name (sdb_get url "select first, last
                           from mail
                           where mail = '" address "'")))
  (cond ((null? name) (echo address ": Unknown user\n"))
        (t (echo address ": " (car name) " " (cadr name) "\n")))))

(mailuser "billg@microsoft.com")
(mailuser "ulric@siag.nu")

; Finally close the database connection
(sdb_close url)

